(* Delphi Dialog
   Auswahl von Filterbedingungen (siehe Unit TextFilter)
   =====================================================
   
   untersttzte Filtertypen:
   - Standard-Delphi (function MatchesMask)
   - Regulre Ausdrcke
   
    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
   
   Vers. 1 - Dez. 2004
   letzte nderung: Sep. 2008
   *)

unit FilterDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls,
  TextFilter, HListBox;

const
  MExp1 = '*';
  MExp2 = '[ab]*';
  RExp1 = '\D';
  RExp2 = 'A+[bc].';

type
  TFilterDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    gbFilter: TGroupBox;
    lbMask: TLabel;
    IncMaskBtn: TRadioButton;
    ExcMaskBtn: TRadioButton;
    NoMaskBtn: TRadioButton;
    RegExpCB: TCheckBox;
    cbxFilter: THistoryCombo;
    btnEditMask: TBitBtn;
    procedure NoMaskBtnClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure btnEditMaskClick(Sender: TObject);
  private
    { Private declarations }
    FFilter : TTextFilter;
  public
    { Public declarations }
    procedure LoadFromIni(IniName,IniSection : string);
    function Execute (Titel,Desc : string; ShowListBtn : boolean;
                      var AFilter : TTextFilter) : boolean;
  end;

procedure InitFilterDialog(const AIniName,ASection : string);
function EditFilter (Titel,Desc : string; ShowListBtn : boolean;
                     var AFilter : TTextFilter) : boolean;

var
  FilterDialog: TFilterDialog;

{ ---------------------------------------------------------------- }
implementation

{$R *.DFM}

uses StringUtils, WinUtils, GnuGetText, SelectFromListDlg;

var
  IniFileName,SectionName   : string;

{ ---------------------------------------------------------------- }
procedure TFilterDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  cbxFilter.Style:=csSimple;
  end;

procedure TFilterDialog.LoadFromIni(IniName,IniSection : string);
begin
  cbxFilter.LoadFromIni(IniName,IniSection);
  end;

procedure TFilterDialog.FormShow(Sender: TObject);
begin
  with cbxFilter do begin
    if Items.Count=0 then Style:=csSimple else Style:=csDropDown;
    Enabled:=not NoMaskBtn.Checked;
    end;
  NoMaskBtnClick(self);
  end;

{ ---------------------------------------------------------------- }
procedure TFilterDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if (ModalResult=mrOK) then with FFilter do begin
    with cbxFilter do if Text='' then begin
      Regular:=false;
      Text:=MatchAll;
      end
    else begin
      Regular:=RegExpCB.Checked;
      end;
    if IncMaskBtn.Checked then Mode:=fmInclude
    else if ExcMaskBtn.Checked then Mode:=fmExclude
    else Mode:=fmOff;
    if not NoMaskBtn.Checked then begin
      try
        Filter:=cbxFilter.Text;
        with cbxFilter do AddItem(Text);
      except
        CanClose:=false;
        if Regular then ErrorDialog(Caption,
          Format(dgettext('dialogs','Invalid filter mask!'+sLineBreak+
                          'Should resemble %s or %s'),
          [RExp1,RExp2]),CenterPos)
        else ErrorDialog(Caption,
          Format(dgettext('dialogs','Invalid filter mask!'+sLineBreak+
                          'Should resemble %s or %s'),
          [MExp1,MExp2]),CenterPos);
        end;
      end;
    end;
  end;

procedure TFilterDialog.NoMaskBtnClick(Sender: TObject);
begin
  cbxFilter.Enabled:=not NoMaskBtn.Checked;
  btnEditMask.Enabled:=cbxFilter.Enabled;
  RegExpCB.Enabled:=not NoMaskBtn.Checked;
  lbMask.Enabled:=RegExpCB.Enabled;
  if Visible and not NoMaskBtn.Checked then cbxFilter.SetFocus;
  end;

procedure TFilterDialog.btnEditMaskClick(Sender: TObject);
var
  s  : string;
  p  : TPoint;
  ml : TStringList;
begin
  s:='';
  ml:=TStringList.Create;
  with ml do begin
    Delimiter:=defFilterSeparator;
    DelimitedText:=cbxFilter.Text;
    end;
  with btnEditMask do p:=ClientToScreen(Point(-20,Height+5));
  if EditList(p,Caption,lbMask.Caption,'',true,false,false,0,tcLower,cbxFilter.Text,ml,s) then begin
    cbxFilter.Text:=ml.DelimitedText;
    ml.Free;
    end;
  end;

function TFilterDialog.Execute (Titel,Desc : string; ShowListBtn : boolean;
                                var AFilter: TTextFilter) : boolean;
begin
  Caption:=Titel;
  gbFilter.Caption:=Desc;
  btnEditMask.Visible:=ShowListBtn;
  FFilter:=AFilter;
  with FFilter do begin
    cbxFilter.Text:=Filter;
    RegExpCB.Checked:=Regular;
    case Mode of
    fmInclude : IncMaskBtn.Checked:=true;
    fmExclude : ExcMaskBtn.Checked:=true;
      else NoMaskBtn.Checked:=true;
      end;
    end;
  Result:=ShowModal=mrOK;
  end;

procedure InitFilterDialog (const AIniName,ASection : string);
begin
  IniFileName:=AIniName; SectionName:=ASection;
  end;

function EditFilter (Titel,Desc : string; ShowListBtn : boolean;
                     var AFilter : TTextFilter) : boolean;
begin
  if not assigned(FilterDialog) then begin
    FilterDialog:=TFilterDialog.Create(Application);
    FilterDialog.LoadFromIni(IniFileName,SectionName);
    end;
  Result:=FilterDialog.Execute(Titel,Desc,ShowListBtn,AFilter);
  FreeAndNil(FilterDialog);
  end;

end.
