(*  Delphi Dialog
    FTP-Verzeichnis auswhlen
    =========================
    
    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Apr. 2007
   *)
    
unit FtpDirDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ImgList,
  FtStrings, FtpFileTools, Vcl.ComCtrls;

type
  TFtpDirDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Label1: TLabel;
    edBaseDir: TEdit;
    Label2: TLabel;
    tvSubDirs: TTreeView;
    ImageList: TImageList;
    edSubDir: TEdit;
    Label3: TLabel;
    procedure tvSubDirsClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private-Deklarationen }
    FTP    : TExtFTP;
    procedure BuildDirTree (Dir : string; Node : TTreeNode);
  public
    { Public-Deklarationen }
    function Execute (AHost,AUsername,ADirectory : string; APassword : AnsiString;
                      APort : integer; APassive : boolean; var ASubDir : string) : boolean;
  end;

function SelectFtpDir (AHost,AUsername,ADirectory : string; APassword : AnsiString;
                       APort : integer; APassive : boolean; var ASubDir : string) : boolean;

var
  FtpDirDialog: TFtpDirDialog;

implementation

uses PwdDlg, WinUtils, UnitStrings, StatWind, GnuGetText;

{$R *.dfm}

{------------------------------------------------------------------- }
procedure TFtpDirDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TFtpDirDialog.BuildDirTree(Dir : string; Node : TTreeNode);
var
  i    : integer;
  dl   : TDirectoryList;
  nd   : TTreeNode;
  base,
  ok   : boolean;
  fds  : string;
begin
  dl:=TDirectoryList.Create;
  ok:=true; base:=length(Dir)=0;
  try fds:=Ftp.GetCurrentDir; except fds:=''; end;
  if not base then begin
    try Ftp.ChangeDir(ExtractFilename(Dir)); except ok:=false; end;
    end;
  if ok and Ftp.GetDirList(dl) then begin
    nd:=Node;
    with dl do for i:=0 to Count-1 do begin
      if base then nd:=tvSubDirs.Items.Add(nd,Strings[i])
      else nd:=tvSubDirs.Items.AddChild(Node,Strings[i]);
      with nd do begin
        ImageIndex:=0;
        SelectedIndex:=1;
        end;
      BuildDirTree(Strings[i],nd);
      Application.ProcessMessages;
      end;
    end;
  Ftp.ChangeToDir(fds);
  dl.Free;
  end;

procedure TFtpDirDialog.tvSubDirsClick(Sender: TObject);
var
  s    : string;
  nd   : TTreeNode;
begin
  nd:=TvSubDirs.Selected;
  s:=nd.Text;
  while nd.Parent<>nil do begin
    nd:=nd.Parent;
    s:=nd.Text+'/'+s;
    end;
  edSubDir.Text:=s;
  end;

{------------------------------------------------------------------- }
function TFtpDirDialog.Execute (AHost,AUsername,ADirectory : string; APassword : AnsiString;
                                APort : integer; APassive : boolean; var ASubDir : string) : boolean;
var
  ok : boolean;
  i  : integer;
begin
  Ftp:=TExtFtp.Create(self);
  with Ftp do begin
    Host:=AHost;
    Port:=APort;
    Username:=AUsername;
    Passive:=APassive;
    TransferTimeout:=defTimeout;
    if length(APassword)=0 then begin
      ok:=ReadPassword(rsFtpConn,Format(rsFtpHint,[Host,Port,Username]),false,false,APassword,i)=mrOK;
      end
    else ok:=true;
    if ok then begin
      edBaseDir.Text:='ftp://'+Host+':'+IntToStr(Port)+'/'+ADirectory;
      StatusWindow.ShowStatus (CenterPos,rsFtpConn,rsTryConnect+': '+Host,true,10);
      Application.ProcessMessages;
      Password:=APassword;
      try Connect; except ok:=false; end;
      if ok then ok:=ForceDir(ADirectory)
      else ErrorDialog(Caption,rsConnectErr);
      StatusWindow.Status:=dgettext('dialogs','Searching for directories');
      tvSubDirs.Items.Clear;
      try
        BuildDirTree('',nil);
      finally
        Ftp.Disconnect;
        StatusWindow.Close;
        end;
      end;
    if ok and (ShowModal=mrOK) then begin
      ASubDir:=edSubDir.Text;
      Result:=true;
      end
    else Result:=false;
    end;
  Ftp.Free;
  end;

{------------------------------------------------------------------- }
function SelectFtpDir (AHost,AUsername,ADirectory : string; APassword : AnsiString;
                       APort : integer; APassive : boolean; var ASubDir : string) : boolean;
begin
  if not assigned(FtpDirDialog) then FtpDirDialog:=TFtpDirDialog.Create(Application);
  Result:=FtpDirDialog.Execute(AHost,AUsername,ADirectory,APassword,APort,APassive,ASubDir);
  FreeAndNil(FtpDirDialog);
  end;

end.
