(* Delphi Dialog
   Seiteneinstellungen fr Ausdruck
   ================================

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Apr. 2009
   *)

unit PrintPageDlg;

interface

uses System.Types, System.Classes, Vcl.Graphics, Vcl.Forms, Vcl.Controls, Vcl.Buttons,
  Vcl.StdCtrls, NumberEd, Vcl.Printers, Vcl.ExtCtrls;

type
  TPrinterSettings = record
    Margins  : TRect;     // in 1/10 mm
    Feed     : integer;   // in 1/10 mm
    FontName : string;
    FontSize : integer;
    Orientation : TPrinterOrientation;
    end;

  TPrintPageDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    GroupBox1: TGroupBox;
    Label1: TLabel;
    ObenEdit: TFloatEdit;
    Label8: TLabel;
    Label2: TLabel;
    UntenEdit: TFloatEdit;
    Label3: TLabel;
    Label5: TLabel;
    Label7: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    EinzugLabel: TLabel;
    EinzugEinheit: TLabel;
    EinzugEdit: TFloatEdit;
    LinksEdit: TFloatEdit;
    RechtsEdit: TFloatEdit;
    rgOrientation: TRadioGroup;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    function Execute (ATitle     : string;
                      ShowEzg    : boolean;
                      var Page   : TPrinterSettings) : boolean; overload;
    function Execute (ATitle     : string;    // in 1/10 mm
                      var Page   : TPrinterSettings) : boolean; overload;
  end;

function EditPrintPageSettings (ATitle     : string;    // in 1/10 mm
                                var Page   : TPrinterSettings) : boolean;

var
  PrintPageDialog: TPrintPageDialog;

implementation

{$R *.DFM}

uses GnuGetText, System.SysUtils;

{------------------------------------------------------------------- }
procedure TPrintPageDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

{------------------------------------------------------------------- }
function TPrintPageDialog.Execute (ATitle     : string;
                                   ShowEzg    : boolean;
                                   var Page   : TPrinterSettings) : boolean;
begin
  if length(ATitle)>0 then Caption:=ATitle;
  if ShowEzg then begin
    ClientHeight:=216;
    EinzugEdit.Show; EinzugLabel.Show; EinzugEinheit.Show;
    end
  else begin
    ClientHeight:=186;
    EinzugEdit.Hide; EinzugLabel.Hide; EinzugEinheit.Hide;
    end;
  with Page do begin
    rgOrientation.ItemIndex:=integer(Orientation);
    with Margins do begin
      ObenEdit.Value:=Top/100;
      UntenEdit.Value:=Bottom/100;
      LinksEdit.Value:=Left/100;
      RechtsEdit.Value:=Right/100;
      end;
    EinzugEdit.value:=Feed/100;
    end;
  ActiveControl:=ObenEdit;
  if ShowModal=mrOK then begin
    with Page do begin
      Orientation:=TPrinterOrientation(rgOrientation.ItemIndex);
      with Margins do begin
        Top:=round(ObenEdit.Value*100);
        Bottom:=round(UntenEdit.Value*100);
        Left:=round(LinksEdit.Value*100);
        Right:=round(RechtsEdit.Value*100);
        end;
      Feed:=round(EinzugEdit.Value*100);
      end;
    Result:=true;
    end
  else Result:=false;
  end;

function TPrintPageDialog.Execute (ATitle   : string;
                                   var Page   : TPrinterSettings) : boolean;
begin
  Result:=Execute (ATitle,false,Page);
  end;

{------------------------------------------------------------------- }
function EditPrintPageSettings (ATitle     : string;    // in 1/10 mm
                                var Page   : TPrinterSettings) : boolean;
begin
  if not assigned(PrintPageDialog) then PrintPageDialog:=TPrintPageDialog.Create(Application);
  Result:=PrintPageDialog.Execute(ATitle,Page);
  FreeAndNil(PrintPageDialog);
  end;

end.
