(*  Delphi Dialog
    Zusamnstellung einer Suchpfad-Liste
    ===================================
    
    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Okt. 2004
   *)
   
unit SearchPathDlg;

interface

uses  Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls;

const
  CvUp  = 1;
  CvOn  = 2;
  DefRowCount = 21;

type
  TSearchPathDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    DirBtn: TSpeedButton;
    Liste: TListBox;
    ReplBtn: TBitBtn;
    PathEdit: TLabeledEdit;
    InsertBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure InsertBtnClick(Sender: TObject);
    procedure EditKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure EditKeyPress(Sender: TObject; var Key: Char);
    procedure DirBtnClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ListeClick(Sender: TObject);
    procedure ReplBtnClick(Sender: TObject);
    procedure ListeDrawItem(Control: TWinControl; Index: Integer; Rect: TRect;
      State: TOwnerDrawState);
  private
    { Private declarations }
    APath : string;
  public
    { Public declarations }
    function Execute (Title,StdPath : string;
                      var AListe : string) : boolean;
  end;

var
  SearchPathDialog: TSearchPathDialog;

implementation

uses ShellDirDlg, GnuGetText;


{$R *.DFM}

procedure TSearchPathDialog.FormCreate(Sender: TObject);
begin
  TranslateComponent (self,'dialogs');
  end;

procedure TSearchPathDialog.UpBtnClick(Sender: TObject);
var
  s  : string;
begin
  with Liste do if Items.Count>1 then begin
    if ItemIndex>0 then with Items do begin
      s:=Strings[ItemIndex];
      Strings[ItemIndex]:=Strings[ItemIndex-1];
      Strings[ItemIndex-1]:=s;
      ItemIndex:=ItemIndex-1;
      end;
    end;
  PathEdit.SetFocus;
  end;

procedure TSearchPathDialog.DownBtnClick(Sender: TObject);
var
  s  : string;
begin
  with Liste do if Items.Count>1 then begin
    if ItemIndex<Items.Count-1 then with Items do begin
      s:=Strings[ItemIndex];
      Strings[ItemIndex]:=Strings[ItemIndex+1];
      Strings[ItemIndex+1]:=s;
      ItemIndex:=ItemIndex+1;
      end;
    end;
  PathEdit.SetFocus;
  end;

procedure TSearchPathDialog.DeleteBtnClick(Sender: TObject);
begin
  with Liste do if ItemIndex>=0 then Items.Delete(ItemIndex);
  PathEdit.SetFocus;
  end;

procedure TSearchPathDialog.InsertBtnClick(Sender: TObject);
begin
  with PathEdit do if length(Trim(Text))>0 then Liste.Items.Add(Text);
  PathEdit.SetFocus;
  end;

procedure TSearchPathDialog.ReplBtnClick(Sender: TObject);
begin
  with PathEdit,Liste do if (ItemIndex>=0) and (length(Trim(Text))>0) then Items[ItemIndex]:=Text;
  PathEdit.SetFocus;
  end;

procedure TSearchPathDialog.EditKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key=VK_RETURN) then InsertBtnClick(Sender);
  end;

procedure TSearchPathDialog.EditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key=#$0D then Key:=#0;
  end;

procedure TSearchPathDialog.DirBtnClick(Sender: TObject);
var
   s : string;
begin
  if length(PathEdit.Text)=0 then s:=APath else s:=PathEdit.Text;
  if ShellDirDialog.Execute (Caption,false,true,APath,s) then PathEdit.Text:=s;
  end;

procedure TSearchPathDialog.ListeClick(Sender: TObject);
begin
  with Liste do if (Items.Count>0) and (ItemIndex>=0) then PathEdit.Text:=Items[ItemIndex];
  end;

procedure TSearchPathDialog.ListeDrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
var
  ok : boolean;
begin
  with (Control as TListBox),Canvas do begin
    ok:=DirectoryExists(Items[Index]);
    if odSelected in State then begin
      Brush.Color:=clHighlight;
      with Font do if ok then Color:=clHighlightText else Color:=clGrayText;
      end
    else begin
      Brush.Color:=clWindow;
      with Font do if ok then Color:=clWindowText else Color:=clGrayText;
      end;
    FillRect(Rect);
    TextOut(Rect.Left,Rect.Top,Items[Index]);
    end;
  end;

{ ------------------------------------------------------------------- }
function TSearchPathDialog.Execute (Title,StdPath : string;
                              var AListe : string) : boolean;
var
  ok : boolean;
  i  : integer;
  s  : string;
begin
  if Title='' then Caption:=Title;
  APath:=StdPath; s:=AListe;
  with Liste do begin
    Clear;
    PathEdit.Text:='';
    if length(s)>0 then begin
      repeat
        i:=pos(';',s);
        if i=0 then i:=succ(length(s));
        Items.Add(copy(s,1,pred(i)));
        delete (s,1,i);
        until (length(s)=0);
      PathEdit.Text:=Items[0];
      end
    else PathEdit.Text:='';
    end;
  ActiveControl:=PathEdit;
  ok:=ShowModal=mrOK;
  if ok then begin
    s:='';
    with Liste.Items do for i:=0 to Count-1 do s:=s+Strings[i]+';';
    delete (s,length(s),1);
    AListe:=s;
    end;
  Result:=ok;
  end;

end.
