(* Delphi Dialog
   Download file from URL and show Progress
   =============================

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2008
   *)

unit DownloadProgress;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms, 
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, Vcl.ComCtrls, 
  IdComponent, IdHttp;

type
  TDownload = class(TForm)
    edURL: TLabeledEdit;
    edFilename: TLabeledEdit;
    pbDownload: TProgressBar;
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    nMax      : int64;
    procedure StartWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
    procedure ShowWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
  public
    { Public declarations }
    function Execute (const URL,Filename : string) : boolean;
  end;

var
  Download: TDownload;

implementation

{$R *.dfm}

procedure TDownload.FormCreate(Sender: TObject);
begin
  // ***
  end;


procedure TDownload.StartWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if AWorkMode=wmRead then nMax:=AWorkCount;
  pbDownload.Position:=0;
  Application.ProcessMessages;
  end;

procedure TDownload.ShowWork(Sender: TObject; AWorkMode: TWorkMode; AWorkCount: Int64);
begin
  if (AWorkMode=wmRead) and (nMax>0) then
    pbDownload.Position:=round((AWorkCount/nMax)*100);
  Application.ProcessMessages;
  end;

function TDownload.Execute (const URL,Filename : string) : boolean;
var
  fs      : TFileStream;
  IdHTTP  : TIdHTTP;
begin
  edURL.Text:=URL;
  edFilename.Text:=Filename;
  IdHTTP:=TIdHTTP.Create(Application);
  with IdHTTP do begin
    OnWorkBegin:=StartWork;
    OnWork:=ShowWork;
    end;
  Result:=false;
  Show;
  BringToFront;
  try
    fs:=TFileStream.Create(Filename,fmCreate);
    try
      IdHTTP.Get(Url,fs);
      Result:=true;
    except
      end;
  finally
    fs.Free;
    end;
  IdHTTP.Free;
  Close;
  end;

end.
