(* Delphi Dialog
   Texteingabe mit Maskenvorgabe
   =============================

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - Jul. 2008
   *)

unit InpMaskedText;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls, HListBox, Vcl.Mask;

type
  TInputMaskedTextDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Descriptor: TLabel;
    TextFeld: TMaskEdit;
    procedure FormCreate(Sender: TObject);
    procedure TextFeldKeyPress(Sender: TObject; var Key: Char);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    { Private declarations }
    FIniName,FIniSection : string;
  public
    { Public declarations }
  function Execute (Pos        : TPoint;
                    Titel,Desc,
                    Mask       : string;
                    var AText  : string) : boolean;
  end;

(* Text eingeben, Ergebnis: "true" bei "ok" *)
function InputMaskedText(Pos        : TPoint;
                         Titel,Desc,
                         Mask       : string;
                         var AText  : string) : boolean;

var
  InputMaskedTextDialog: TInputMaskedTextDialog;

implementation

{$R *.DFM}

uses CharTableDlg, System.IniFiles, WinUtils;

procedure TInputMaskedTextDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if Modalresult=mrCancel then TextFeld.EditMask:='';
  end;

procedure TInputMaskedTextDialog.FormCreate(Sender: TObject);
begin
  // ***
  end;

{ ------------------------------------------------------------------- }
procedure TInputMaskedTextDialog.TextFeldKeyPress(Sender: TObject;
  var Key: Char);
begin
  if Key=#$1B then Modalresult:=mrCancel
  else if Key=#$D then Modalresult:=mrOK;
  end;

{ ------------------------------------------------------------------- }
function TInputMaskedTextDialog.Execute (Pos        : TPoint;
                                   Titel,Desc,
                                   Mask       : string;
                                   var AText  : string) : boolean;
begin
  with Pos do begin
    if (Y < 0) or (X < 0) then Position:=poScreenCenter
    else begin
      Position:=poDesigned;
      if X<0 then X:=Left;
      if Y<0 then Y:=Top;
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=x; Top:=y;
      end;
    end;
  Caption:=Titel;
  Descriptor.Caption:=Desc;
  ActiveControl:=TextFeld;
  with TextFeld do begin
    Text:=AText;
    EditMask:=Mask;
    end;
  if ShowModal=mrOK then begin
    AText:=TextFeld.Text;
    Result:=true;
    end
  else Result:=false;
  end;

{ ------------------------------------------------------------------- }
(* Txt eingeben, Ergebnis: "true" bei "ok" *)
function InputMaskedText(Pos        : TPoint;
                         Titel,Desc,
                         Mask       : string;
                         var AText  : string) : boolean;
begin
  if not assigned(InputMaskedTextDialog) then
    InputMaskedTextDialog:=TInputMaskedTextDialog.Create(Application);
  with InputMaskedTextDialog do begin
    Result:=Execute(Pos,Titel,Desc,Mask,AText);
    Release;
    end;
  InputMaskedTextDialog:=nil;
  end;

end.
