(* Dialog for Mail connection parameters

    Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.

   Vers. 1 - May 2007
   *)

unit MailDlg;

interface

uses Winapi.Windows, System.SysUtils, System.Classes, Vcl.Graphics, Vcl.Forms,
  Vcl.Controls, Vcl.StdCtrls, Vcl.Buttons, Vcl.ExtCtrls,
  HListBox, NumberEd, IdAssignedNumbers;

const
  defSmtpPort = IdPORT_SMTP;

type
  TMailMode = (mmNone,mmAlways,mmError);

  TMailData = record
    SMTP,
    Username,
    Password,
    Sender,
    ReplyTo,
    Recipients,
    Subject       : string;
    Port,
    SecureMode    : integer;
    Enabled       : boolean;
    end;

  TMailDialog = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    lbRecipients: TListBox;
    Label5: TLabel;
    Label6: TLabel;
    edPwd: TEdit;
    btShowPwd: TSpeedButton;
    edServer: THistoryCombo;
    edUsername: THistoryCombo;
    edSender: TComboBox;
    edReply: TComboBox;
    Label7: TLabel;
    edPort: TRangeEdit;
    Panel1: TPanel;
    Panel2: TPanel;
    Panel3: TPanel;
    btnAdd: TBitBtn;
    btnRemove: TBitBtn;
    btnEdit: TBitBtn;
    edSubject: TLabeledEdit;
    gbSecure: TGroupBox;
    rbNone: TRadioButton;
    rbAvail: TRadioButton;
    rbTLS: TRadioButton;
    rbSSL: TRadioButton;
    cbMail: TCheckBox;
    procedure btnAddClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure btnEditClick(Sender: TObject);
    procedure btShowPwdClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure rbNoneClick(Sender: TObject);
    procedure rbAvailClick(Sender: TObject);
    procedure rbTLSClick(Sender: TObject);
    procedure rbSSLClick(Sender: TObject);
  private
    { Private declarations }
    FIniName : string;
    hl       : TStringList;
    fSecureMode : integer;
  public
    { Public declarations }
    function DefaultValues : TMailData;
    procedure LoadFromIni (const AIniName : string);
    function Execute (ShowPwd,Secure : boolean; const Prompt : string;
                          var AMailDat : TMailData) : boolean;
  end;

var
  MailDialog: TMailDialog;

implementation

{$R *.DFM}

uses Vcl.Dialogs, WinUtils, System.IniFiles, InpText, mMailer10;

const
  SmptSekt = 'SmtpServer';
  UserSekt = 'MailUser';
  MailSekt = 'MailTo';

  iniMName = 'Address';

procedure TMailDialog.FormCreate(Sender: TObject);
begin
  FIniName:='';
  hl:=TStringList.Create;
  end;

procedure TMailDialog.FormDestroy(Sender: TObject);
var
  IniFile : TIniFile;
begin
  if length(FIniName)>0 then begin
    IniFile:=TIniFile.Create(FIniName);
    SaveHistory(IniFile,MailSekt,iniMName,true,hl);
    IniFile.Free;
    end;
  hl.Free;
  end;

procedure TMailDialog.LoadFromIni (const AIniName : string);
var
  IniFile : TIniFile;
begin
  // nur beim Neustart laden
  if (length(FIniName)=0) and (length(AIniName)>0) then begin
    FIniName:=AIniName;
    edServer.LoadFromIni(FIniName,SmptSekt);
    edUserName.LoadFromIni(FIniName,UserSekt);
    IniFile:=TIniFile.Create(FIniName);
    LoadHistory(IniFile,MailSekt,iniMName,hl);
    IniFile.Free;
    end;
  end;

procedure TMailDialog.btnAddClick(Sender: TObject);
var
  t : string;
begin
  t:='';
  if InputText(CursorPos,'Neuer Mailempfnger','Adresse:',false,'',hl,false,15,t) then
    with lbRecipients do if Items.IndexOf(t)<0 then ItemIndex:=Items.Add(t);
  end;

procedure TMailDialog.btnRemoveClick(Sender: TObject);
var
  n : integer;
begin
  with lbRecipients do if (Items.Count>0) and (ItemIndex>=0) then begin
    n:=ItemIndex;
    if (ItemIndex>=0) and
        ConfirmDialog(Caption,
          Format('Mailempfnger:'+sLineBreak+'%s'+sLineBreak+'aus Liste entfernen?',
          [Items[ItemIndex]])) then begin
      Items.Delete(ItemIndex);
      if n>=Items.Count then n:=Items.Count-1;
      ItemIndex:=n;
      end;
    end;
  end;

procedure TMailDialog.btnEditClick(Sender: TObject);
var
  t : string;
begin
  with lbRecipients do if (Items.Count>0) and (ItemIndex>=0) then begin
    t:=Items[ItemIndex];
    if InputText(CursorPos,'Mailempfnger bearbeiten','Adresse:',false,'',hl,false,15,t) then
      Items[ItemIndex]:=t;
    end;
  end;

procedure TMailDialog.btShowPwdClick(Sender: TObject);
var
  c : char;
begin
  if btShowPwd.Down then c:=#0 else c:='*';
  edPwd.PasswordChar:=c;
  end;

procedure TMailDialog.rbNoneClick(Sender: TObject);
begin
  fSecureMode:=0;
  edPort.Value:=IdPORT_SMTP;
  end;

procedure TMailDialog.rbAvailClick(Sender: TObject);
begin
  fSecureMode:=1;
  edPort.Value:=IdPORT_SMTP;
  end;

procedure TMailDialog.rbTLSClick(Sender: TObject);
begin
  fSecureMode:=2;
  edPort.Value:=IdPORT_SMTP;
  end;

procedure TMailDialog.rbSSLClick(Sender: TObject);
begin
  fSecureMode:=3;
  edPort.Value:=IdPORT_ssmtp;
  end;

{ ---------------------------------------------------------------- }
function TMailDialog.DefaultValues : TMailData;
begin
  with Result do begin
    SMTP:=''; Port:=defSmtpPort;
    Sender:=''; ReplyTo:=''; Recipients:='';
    Username:=''; Password:=''; Subject:='';
    Enabled:=false;
    end;
  end;

function TMailDialog.Execute (ShowPwd,Secure : boolean;  const Prompt : string;
                              var AMailDat : TMailData) : boolean;
begin
  btShowPwd.Visible:=ShowPwd;
  gbSecure.Visible:=Secure;
  with cbMail do begin
    Caption:=Prompt;
    Visible:=length(Prompt)>0;
    end;
  if Secure then ClientHeight:=417
  else ClientHeight:=371;
  with AMailDat do begin
    edServer.Text:=SMTP;
    edUsername.Text:=Username;
    edPwd.Text:=Password;
    with edSender do begin
      Items.Assign(hl);
      Text:=Sender;
      end;
    with edReply do begin
      Items.Assign(hl);
      Text:=ReplyTo;
      end;
    lbRecipients.Items.CommaText:=Recipients;
    fSecureMode:=SecureMode;
    case SecureMode of
    1 : rbAvail.Checked:=true;
    2 : rbTLS.Checked:=true;
    3 : rbSSL.Checked:=true;
    else rbNone.Checked:=true;
      end;
    edPort.Value:=Port;
    edSubject.Text:=Subject;
    cbMail.Checked:=Enabled;
    end;
  if ShowModal=mrOK then with AMailDat do begin
    SMTP:=edServer.Text;
    Port:=edPort.Value;
    Username:=edUsername.Text;
    Password:=edPwd.Text;
    Sender:=edSender.Text;
    AddToHistory(hl,Sender);
    ReplyTo:=edReply.Text;
    AddToHistory(hl,ReplyTo);
    Recipients:=lbRecipients.Items.CommaText;
    SecureMode:=fSecureMode;
    Subject:=edSubject.Text;
    Enabled:=cbMail.Checked;
    Result:=true;
    end
  else Result:=false;
  end;

procedure TMailDialog.FormCloseQuery(Sender: TObject;
  var CanClose: Boolean);
begin
  if ModalResult=mrOK then begin
    if length(edServer.Text)=0 then begin
      CanClose:=false;
      ErrorDialog('Bitte geben Sie einen SMTP-Server an!');
      edServer.SetFocus;
      end
    else if lbRecipients.Items.Count=0 then begin
      CanClose:=false;
      ErrorDialog('Bitte geben Sie mindestens einen Empfnger an!');
      btnAdd.Click;
      end;
    end;
  end;

end.
