(* Delphi Dialog
   Anzeige einer Log-Datei (z.B. von Personal Backup)
   ==================================================
   Textanzeige mit TMemo
   Suchfunktionen fr 2 spez. Textteile
   Drucken (auch von markierten Textteilen)
   
    2004-2015, Dr. J. Rathlev, D-24222 Schwentinental (info(a)rathlev-home.de)

   The contents of this file may be used under the terms of the
   Mozilla Public License ("MPL") or
   GNU Lesser General Public License Version 2 or later (the "LGPL")

   Software distributed under this License is distributed on an "AS IS" basis,
   WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
   the specific language governing rights and limitations under the License.
    
   Vers. 1 - Sep 2004
   *)


unit ShowMemo;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.Buttons,
  Vcl.Printers, Vcl.ComCtrls, System.Math, PrintPageDlg;

const
  defTopMargin = 150;  // Angaben in 1/10 mm
  defBottomMargin= 150;
  defLeftMargin = 200;
  defRightMargin= 100;
  defFontName = 'Courier New';
  defFontSize = 10;
  defOrientation = poPortrait;

type
  TShowDlgBtn = (sbOpen,sbErase,sbPrint,sbSearch,sbSection,sbError);
  TShowDlgButtons = set of TShowDlgBtn;

  TShowDlgType = (stShow,stModal);

  TShowTextDialog = class(TForm)
    Panel2: TPanel;
    EndeBtn: TBitBtn;
    PrintBtn: TBitBtn;
    PrintDialog: TPrintDialog;
    DeleteBtn: TBitBtn;
    FindDialog: TFindDialog;
    SearchBtn: TBitBtn;
    NextSectBtn: TBitBtn;
    PrevErrBtn: TBitBtn;
    PrevSectBtn: TBitBtn;
    NextErrBtn: TBitBtn;
    StatusBar: TStatusBar;
    Panel1: TPanel;
    Memo: TMemo;
    OpenBtn: TBitBtn;
    OpenDialog: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure MemoKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure PrintBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure MemoKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormActivate(Sender: TObject);
    procedure MemoChange(Sender: TObject);
    procedure MemoMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FindDialogFind(Sender: TObject);
    procedure SearchBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure NextSectBtnClick(Sender: TObject);
    procedure PrevErrBtnClick(Sender: TObject);
    procedure PrevSectBtnClick(Sender: TObject);
    procedure NextErrBtnClick(Sender: TObject);
    procedure OpenBtnClick(Sender: TObject);
    procedure EndeBtnClick(Sender: TObject);
  private
    { Private-Deklarationen }
    LPos             : integer;
    FIniName,FIniSection,
    FTitle,FFilter,
    SectText,ErrText : string;
    FDlgType : TShowDlgType;
    FCodePage  : integer;
    procedure Print (NoPrompt,SelectedLines : boolean);
    function LoadText (const FName : string) : boolean;
  public
    { Public-Deklarationen }
    TextName : string;
    procedure Execute (const Title,TextDatei,
                       PrevCap1,NextCap1,SrchText1,
                       PrevCap2,NextCap2,SrchText2,
                       Filter                   : string;
                       Position  : TPoint;
                       Line      : integer;
                       DlgType   : TShowDlgType;
                       Buttons   : TShowDlgButtons;
                       CodePage  : integer = 0);
    end;

procedure LoadPrinterSettings(const AIniName,ASection : string);
procedure SavePrinterSettings;
procedure PrintTextFile (const TextDatei: string; CodePage : integer = 0);
procedure ShowText (const Title,TextDatei,
                     PrevCap1,NextCap1,SrchText1,
                     PrevCap2,NextCap2,SrchText2,
                     Filter                   : string;
                     Position  : TPoint;
                     Line      : integer;
                     DlgType   : TShowDlgType;
                     Buttons   : TShowDlgButtons;
                     CodePage  : integer = 0);
var
  ShowTextDialog: TShowTextDialog;
  PrinterSettings : TPrinterSettings;

implementation

{$R *.DFM}

uses FileUtils, System.IniFiles, WinUtils, System.StrUtils;

var
  IniFileName,SectionName   : string;

{------------------------------------------------------------------- }
// Ersatz fr Bibliotheksfunktion, da dort nur 16-bit-Werte verarbeitet werden
function GetCaretPos (Memo : TMemo) : TPoint;
var
  CPos : integer;
begin
  with Memo do begin
    Result.X := SendMessage(Handle, EM_GETSEL, WParam(@CPos), 0);
    Result.Y := SendMessage(Handle, EM_LINEFROMCHAR, CPos, 0);
    Result.X := CPos - SendMessage(Handle, EM_LINEINDEX, -1, 0);
    end;
  end;


{ ------------------------------------------------------------------- }
function FindOptionsToSearchOptions (FOptions : TFindOptions) : TStringSearchOptions;
begin
  Result:=[];
  if frDown in FOptions then Include(Result,soDown);
  if frMatchCase in FOptions then Include(Result,soMatchCase);
  if frWholeWord in FOptions then Include(Result,soWholeWord);
  end;

function SearchMemo(Memo: TMemo; SearchDown : boolean;
                    const SearchString: String;
                    Options: TFindOptions): Boolean;
var
  Buffer, P : PChar;
  Size      : cardinal;
begin
  Result := False;
  if (Length(SearchString) = 0) then Exit;
  Size := Memo.GetTextLen;
  if (Size = 0) then Exit;
  Buffer := StrAlloc(Size + 1);
  try
    Memo.GetTextBuf(Buffer, Size + 1);
    P := SearchBuf(Buffer, Size, Memo.SelStart, Memo.SelLength, SearchString, FindOptionsToSearchOptions(Options));
    if P <> nil then with memo do begin
      SelStart:= P - Buffer;     // Number of line has to be subtracted to get right value for SelStart ???
      SelLength := Length(SearchString);
      Perform(EM_SCROLLCARET,0,0);
      if SearchDown and (Perform(EM_GETFIRSTVISIBLELINE,0,0)<Perform(EM_LINEFROMCHAR,SelStart,0)) then begin
        Perform(EM_SCROLL,SB_LINEDOWN,0); Perform(EM_SCROLL,SB_LINEDOWN,0);
        end;
      Result := True;
    end;
  finally
    StrDispose(Buffer);
  end;
end;

{ ------------------------------------------------------------------- }
procedure TShowTextDialog.FormCreate(Sender: TObject);
begin
  Memo.Clear;
  with PrinterSettings do begin
    with Margins do begin
      Top:=defTopMargin;
      Bottom:=defBottomMargin;
      Left:=defLeftMargin;
      Right:=defRightMargin;
      end;
    FontName:=defFontName;
    FontSize:=defFontSize;
    Orientation:=defOrientation;
    end;
  FIniName:=''; FIniSection:='';
  end;

{ ---------------------------------------------------------------- }
(* Cursorposition anzeigen *)
procedure TShowTextDialog.MemoChange(Sender: TObject);
var
  pt : TPoint;
begin
  pt:=GetCaretPos(Memo);
  StatusBar.Panels[0].Text:=Format(' Zeile: %u von %u',[pt.y+1,Memo.Lines.Count+1]);
  end;

procedure TShowTextDialog.FormShow(Sender: TObject);
var
  LineNr : integer;
begin
  LoadText(TextName);
  with Memo do begin
    if (LPos<=0) or (LPos>=Lines.Count) then begin
      LineNr:=Lines.Count;
      Perform(WM_VSCROLL,SB_BOTTOM,0);
      end
    else begin
      LineNr:=LPos-1;
      Perform(EM_SCROLLCARET,0,0);
      end;
    SelStart:=Perform(EM_LINEINDEX,LineNr,0);
    SelLength:=0;
//    if LineNr=Lines.Count then SelLength:=0 else SelLength:=Perform(EM_LINEINDEX,LineNr+1,0)-SelStart;
    StatusBar.Panels[0].Text:=Format(' Zeile: %u von %u',[LineNr+1,Lines.Count+1]);
    end;
  with FindDialog do Options:=Options -[frDown];
  end;

procedure TShowTextDialog.FormPaint(Sender: TObject);
begin
  MemoChange(Sender);
  end;

procedure TShowTextDialog.MemoKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key=VK_ESCAPE then ModalResult:=mrCancel;
  end;

procedure TShowTextDialog.MemoKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Shift=[ssCtrl]) and (Key=ord('F')) then SearchBtnClick(Sender);
  if (Shift=[]) and (Key=VK_F3) then FindDialogFind(Sender);
  if (Key=VK_DELETE)  and (Memo.SelLength>0) then begin
    if ConfirmDialog (Caption,'Ausgewhlten Text lschen?',
          Point(Left+200,Top+100)) then begin
      with Memo do begin
        ReadOnly:=false;
        ClearSelection;
        Lines.SaveToFile(TextName);
        ReadOnly:=true;
        end;
      end;
    end;
  MemoChange(Sender);
  end;

procedure TShowTextDialog.FormActivate(Sender: TObject);
begin
  Memo.SetFocus;
  end;

{ ------------------------------------------------------------------- }
type
  TCharRange = record
    cpMin: Longint;
    cpMax: LongInt;
    end;

procedure TShowTextDialog.Print (NoPrompt,SelectedLines : boolean);
var
  w,v       : TSize;
  i,k,lh,
  n1,n2,
  y,lp,hp   : integer;
  txt,s,t   : string;
  Selection : TCharRange;
  f         : double;

const
  PtToCm = 0.0353;   (* 1 Pt in cm *)
  EM_EXGETSEL                         = WM_USER + 52; 

  procedure CheckForNewPage;
  begin
    with Printer,Canvas do begin
      if y>=lp then begin
        y:=PrinterSettings.Margins.Top;
        newpage;
        end;
      if y=PrinterSettings.Margins.Top then begin   // Kopfzeile erzeugen
        font.style := [fsbold];
        txt:=Format('Seite: %u',[pagenumber]);
        textout(PrinterSettings.Margins.Left,y,StripPath(Caption,72));
        textout(hp-TextWidth(txt),y,txt);
        inc(y,lh+3);
        moveto (PrinterSettings.Margins.Left,y); lineto (hp,y);
        font.style := [];
        inc(y,20);
        end;
      end;
    end;

begin
  if NoPrompt or PrintDialog.Execute then begin
    Screen.Cursor:=crHourglass;
    with Printer do begin
      Orientation:=PrinterSettings.Orientation;
      lp:=GetDeviceCaps (Handle,VERTSIZE);
      if lp<=0 then lp:=2900 else lp:=10*lp;  // Seitenhhe in 1/10 mm
      hp:=GetDeviceCaps (Handle,HORZSIZE);
      if hp<=0 then hp:=2000 else hp:=10*hp;  // Seitenbreite in 1/10 mm
      Title:=Caption;
      Begindoc;
      with Canvas do begin
        setmapmode(handle,mm_lometric);     // 1/10 mm aber Y geht nach oben
        GetWindowExtEx(handle,w);           // Auflsung ermitteln
        GetViewportExtEx(handle,v);
        setmapmode(handle,MM_ANISOTROPIC);         // neuer Abb.-Modus
        SetWindowExtEX(handle,w.cx,w.cy,nil);      // Skalierung ...
        SetViewPortExtEx(handle,v.cx,-v.cy,nil);
        f:=PtToCm*100*PrinterSettings.FontSize;    (* Punkte in 0,1 mm umrechnen *)
        lh:=round(f);
        with Font do begin
          Name:=PrinterSettings.FontName;
//          Size:=PrinterSettings.FontSize;
          Height:=lh;
          end;
        Brush.Color := clwhite;   // Hintergrundfarbe
        lp:=lp-PrinterSettings.Margins.Bottom;
        hp:=hp-PrinterSettings.Margins.Right;
        y:=PrinterSettings.Margins.Top;

        with Memo do begin
          if SelectedLines then begin
            (* Zeilennummern des selektierten Bereichs ermitteln *)
            Perform(EM_EXGETSEL,0,longint(@Selection));
            with Selection do begin
              n1:=Perform(EM_LINEFROMCHAR,cpMin,0);
              n2:=Perform(EM_LINEFROMCHAR,cpMax,0);
              end;
            end
          else begin
            n1:=0; n2:=Lines.Count-1;
            end;
          end;
        for i:=n1 to n2 do begin
          CheckForNewPage;
          txt:=Memo.Lines[i];
          while TextWidth(txt)>=hp-PrinterSettings.Margins.Left do begin
            s:='';
            repeat
              k:=Min(Pos('\',txt),Pos(' ',txt));
              if k>0 then begin
                t:=copy(txt,1,k);
                if TextWidth(s+t)<hp-PrinterSettings.Margins.Left then begin
                  s:=s+t; delete(txt,1,k); t:='';
                  end;
                end
              else t:='x';
              until length(t)>0;
            textout(PrinterSettings.Margins.Left,y,s);
            txt:='* '+txt;
            inc(y,lh+3);
            CheckForNewPage;
            end;
          textout(PrinterSettings.Margins.Left,y,txt);
          inc(y,lh+3);
          end;
        end;
      EndDoc;
      end;
    Screen.Cursor:=crDefault;
    end;
  end;

procedure TShowTextDialog.PrintBtnClick(Sender: TObject);
var
  sel      : boolean;
begin
  if not Printer.Printing then begin
    if Memo.SelLength>0 then sel:=ConfirmDialog('Ausgewhlte Zeilen drucken?')
    else sel:=false;
    Print(false,sel);
    Memo.SetFocus;
    end;
  end;

{ ------------------------------------------------------------------- }
procedure TShowTextDialog.DeleteBtnClick(Sender: TObject);
begin
  if Memo.SelLength>0 then begin
    if ConfirmDialog (Caption,'Ausgewhlten Text lschen?',
          Point(Left+200,Top+100)) then begin
      with Memo do begin
        ReadOnly:=false;
        ClearSelection;
        Lines.SaveToFile(TextName,TEncoding.UTF8);
        ReadOnly:=true;
        end;
      end;
    end
  else begin
    if ConfirmDialog (Caption,Format('Datei "%s" lschen?',[TextName]),
          Point(Left+200,Top+100)) then begin
      DeleteFile(TextName);
      Memo.Clear;
      end;
    ModalResult:=mrCancel;
    end;
  end;

procedure TShowTextDialog.MemoMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MemoChange(Sender);
  end;

procedure TShowTextDialog.FindDialogFind(Sender: TObject);
begin
  with FindDialog do
    if not SearchMemo(Memo,false,FindText,Options) then
      ShowMessage(Format('"%s" nicht gefunden!',[FindText]))
    else MemoChange(Sender);
  end;

procedure TShowTextDialog.SearchBtnClick(Sender: TObject);
begin
  FindDialog.Execute;
  end;

procedure TShowTextDialog.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (ssCtrl in Shift) and (Key=ord('F')) then FindDialog.Execute;
  if (Shift=[]) and (Key=VK_F3) then FindDialogFind(Sender);
  end;

procedure TShowTextDialog.PrevSectBtnClick(Sender: TObject);
begin
  with FindDialog do begin
    FindText:=SectText; Options:=Options-[frDown];
    if not SearchMemo(Memo,false,FindText,Options) then begin
      Memo.SelStart:=0;
      StatusBar.Panels[1].Text:='Keine weitereren Abschnitte!';
      end
    else StatusBar.Panels[1].Text:='';
    MemoChange(Sender);
    end;
  end;

procedure TShowTextDialog.NextSectBtnClick(Sender: TObject);
var
  pt : TPoint;
begin
  with FindDialog do begin
    FindText:=SectText; Options:=Options+[frDown];
    if not SearchMemo(Memo,true,FindText,Options) then with Memo do begin
      pt.x:=0; pt.y:=Lines.Count; CaretPos:=pt;
      Perform(EM_SCROLLCARET,0,0);
      StatusBar.Panels[1].Text:='Keine weitereren Abschnitte!';
      end
    else StatusBar.Panels[1].Text:='';
    MemoChange(Sender);
    end;
  end;

function TShowTextDialog.LoadText (const FName : string) : boolean;
var
  fs : TStream;
begin
  TextName:=FName;
  Result:=false;
  with Memo do begin
    fs:= TFileStream.Create(FName,fmOpenRead or fmShareDenyNone);
    try
      if FCodePage>0 then Lines.LoadFromStream(fs,TEncoding.GetEncoding(FCodePage))
      else Lines.LoadFromStream(fs);
      Result:=true;
    finally
      fs.Free;
      end;
    end;
  end;

procedure TShowTextDialog.OpenBtnClick(Sender: TObject);
begin
  with OpenDialog do begin
    if length(TextName)>0 then InitialDir:=ExtractFilePath(TextName)
    else InitialDir:='';
    Title:=FTitle;
    Filter:=FFilter;
    Filename:='';
    if Execute then begin
      LoadText(Filename);
      Caption:=Title+' ['+TextName+']';
      end;
    end;
  end;

procedure TShowTextDialog.PrevErrBtnClick(Sender: TObject);
begin
  with FindDialog do begin
    FindText:=ErrText; Options:=Options-[frDown];
    if not SearchMemo(Memo,false,FindText,Options) then begin
      Memo.SelStart:=0;
      StatusBar.Panels[1].Text:='Keine weitereren Fehler!';
      end
    else StatusBar.Panels[1].Text:='';
    MemoChange(Sender);
    end;
  end;

procedure TShowTextDialog.NextErrBtnClick(Sender: TObject);
var
  pt : TPoint;
begin
  with FindDialog do begin
    FindText:=ErrText; Options:=Options+[frDown];
    if not SearchMemo(Memo,true,FindText,Options) then with Memo do begin
      pt.x:=0; pt.y:=Lines.Count; CaretPos:=pt;
      StatusBar.Panels[1].Text:='Keine weitereren Fehler!';
      end
    else StatusBar.Panels[1].Text:='';
    MemoChange(Sender);
    end;
  end;

{ ------------------------------------------------------------------- }
(* Text anzeigen *)
procedure TShowTextDialog.EndeBtnClick(Sender: TObject);
begin
  if FDlgType=stShow then Close;
  end;

procedure TShowTextDialog.Execute (const Title,TextDatei,
                                   PrevCap1,NextCap1,SrchText1,
                                   PrevCap2,NextCap2,SrchText2,
                                   Filter                   : string;
                                   Position  : TPoint;
                                   Line      : integer;
                                   DlgType   : TShowDlgType;
                                   Buttons   : TShowDlgButtons;
                                   CodePage  : integer = 0);
var
  x  : integer;
begin
  if Visible then BringToFront
  else begin
    with Position do begin
      CheckScreenBounds(Screen,x,y,Width,Height);
      Left:=x; Top:=y;
      end;
    FTitle:=Title;
    Caption:=Title+' ['+TextDatei+']';
    FFilter:=Filter;
    LPos:=Line;
    x:=2;
    with OpenBtn do if sbOpen in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    with PrintBtn do if sbPrint in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      Enabled:=Printer.Printers.Count>0;
      end
    else Visible:=false;
    with DeleteBtn do if sbErase in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    with SearchBtn do if sbSearch in Buttons then begin
      Visible:=true; Left:=x; x:=x+Width-1;
      end
    else Visible:=false;
    if sbSection in Buttons then begin
      SectText:=SrchText1;
      with PrevSectBtn do begin
        Visible:=true; Left:=x; x:=x+Width-1;
        Hint:=PrevCap1;
        end;
      with NextSectBtn do begin
        Visible:=true; Left:=x; x:=x+Width-1;
        Hint:=NextCap1;
        end;
      end
    else begin
      PrevSectBtn.Visible:=false;
      NextSectBtn.Visible:=false;
      end;
    if sbError in Buttons then begin
      ErrText:=SrchText2;
      with PrevErrBtn do begin
        Visible:=true; Left:=x; x:=x+Width-1;
        Hint:=PrevCap2;
        end;
      with NextErrBtn do begin
        Visible:=true; Left:=x;
        Hint:=NextCap2;
        end;
      end
    else begin
      PrevErrBtn.Visible:=false;
      NextErrBtn.Visible:=false;
      end;
    FCodePage:=CodePage;
    FDlgType:=DlgType;
    if FileExists (TextDatei) then begin
//      LoadText(TextDatei);
      TextName:=TextDatei;
      if DlgType=stModal then ShowModal else Show;
      end
    else ErrorDialog ('',Format('Datei: "%s" nicht gefunden!',[TextDatei]));
    end;
  end;

{ ------------------------------------------------------------------- }
const
  IniLeftMarg = 'LeftMargin';
  IniRightMarg = 'RightMargin';
  IniTopMarg = 'TopMargin';
  IniBottomMarg = 'BottomMargin';
  IniFontName = 'FontName';
  IniFontSize = 'FontSize';
  IniOrientation = 'Orientation';

procedure LoadPrinterSettings(const AIniName,ASection : string);
var
  n : integer;
begin
  IniFileName:=AIniName; SectionName:=ASection;
  if (length(IniFileName)>0) and (length(SectionName)>0) then
      with TIniFile.Create(SectionName),PrinterSettings do begin
    with Margins do begin
      Top:=ReadInteger(SectionName,IniTopMarg,defTopMargin);
      Bottom:=ReadInteger(SectionName,IniBottomMarg,defBottomMargin);
      Left:=ReadInteger(SectionName,IniLeftMarg,defLeftMargin);
      Right:=ReadInteger(SectionName,IniRightMarg,defRightMargin);
      end;
    FontName:=ReadString(SectionName,IniFontName,defFontName);
    FontSize:=ReadInteger(SectionName,IniFontSize,defFontSize);
    n:=ReadInteger (SectionName,IniOrientation,ord(Orientation));
    if (n<0) or (n>1) then n:=ord(Orientation);
    Orientation:=TPrinterOrientation(n);
    Free;
    end;
  end;

procedure SavePrinterSettings;
begin
  if (length(IniFileName)>0) and (length(SectionName)>0) then
      with TIniFile.Create(IniFileName),PrinterSettings do begin
    try
      with Margins do begin
        WriteInteger(SectionName,IniTopMarg,Top);
        WriteInteger(SectionName,IniBottomMarg,Bottom);
        WriteInteger(SectionName,IniLeftMarg,Left);
        WriteInteger(SectionName,IniRightMarg,Right);
        end;
      WriteString(SectionName,IniFontName,FontName);
      WriteInteger(SectionName,IniFontSize,FontSize);
      WriteInteger (SectionName,IniOrientation,ord(Orientation));
    finally
      Free;
      end;
    end;
  end;

procedure ShowText (const Title,TextDatei,PrevCap1,NextCap1,SrchText1,PrevCap2,
                    NextCap2,SrchText2,Filter : string;
                    Position : TPoint; Line : integer;
                    DlgType : TShowDlgType; Buttons : TShowDlgButtons;
                    CodePage : integer = 0);
begin
  if not assigned(ShowTextDialog) then ShowtextDialog:=TShowtextDialog.Create(Application);
  ShowtextDialog.Execute(Title,TextDatei,PrevCap1,NextCap1,SrchText1,PrevCap2,
    NextCap2,SrchText2,Filter,Position,Line,DlgType,Buttons,CodePage);
  if DlgType=stModal then FreeAndNil(ShowTextDialog);
  end;

procedure PrintTextFile (const TextDatei : string; CodePage : integer = 0);
begin
  if FileExists (TextDatei) then begin
    if not assigned(ShowTextDialog) then ShowtextDialog:=TShowtextDialog.Create(Application);
    with ShowtextDialog do begin
      if CodePage>0 then Memo.Lines.LoadFromFile(TextDatei,TEncoding.GetEncoding(CodePage))
      else Memo.Lines.LoadFromFile(TextDatei);
      Caption:=ExtractFileName(TextDatei);
      Print(true,false);
      end;
    FreeAndNil(ShowTextDialog);
    end;
  end;

initialization
  IniFileName:=''; SectionName:='';
finalization
end.
